﻿using System;
using System.Configuration;
using System.Diagnostics;

namespace PicklistUpdate {
	static class Program {
		// app.config values
		private static string ServerType = ConfigurationManager.AppSettings["ServerType"];
		private static string ServerName = ConfigurationManager.AppSettings["ServerName"];
		private static string DatabaseName = ConfigurationManager.AppSettings["DatabaseName"];
		private static string DatabaseUserName = ConfigurationManager.AppSettings["DatabaseUserName"];
		private static string DatabasePassword = ConfigurationManager.AppSettings["DatabasePassword"];
		private static string SapUserName = ConfigurationManager.AppSettings["SapUserName"];
		private static string SapPassword = ConfigurationManager.AppSettings["SapPassword"];
		private static string Picklist = ConfigurationManager.AppSettings["Picklist"];

		private static double Qty;

		private static void Main(string[] args) {
			try {
				Console.WriteLine(" Connecting to company...");
				SAPbobsCOM.Company company = ConnectToCompany();
				Console.WriteLine();
				Console.WriteLine(string.Format("   Update Picklist... "));
				CheckParameters();
				UpdateConfiguredPicklist(company);
				Console.WriteLine();
			} catch (Exception ex) {
				Console.WriteLine("   Error: " + ex.Message);
			}
			Console.ReadKey();
			Console.WriteLine();
			Console.Write(" Exiting... ");
		}

		private static void UpdateConfiguredPicklist(SAPbobsCOM.Company company) {
			SAPbobsCOM.PickLists SAPpickList = (SAPbobsCOM.PickLists)company.GetBusinessObject(SAPbobsCOM.BoObjectTypes.oPickLists);
			SAPpickList.GetByKey(Int32.Parse(Picklist));

			Stopwatch stopwatch = Stopwatch.StartNew();
			int result = SAPpickList.Update();
			stopwatch.Stop();
			if (result != 0) {
				Console.WriteLine(string.Format("   Failed (" + company.GetLastErrorDescription() + ")"));
			} else {
				Console.WriteLine("   Successfully done in " + stopwatch.ElapsedMilliseconds + " ms");
			}
		}
		private static void CheckParameters() {
			if (string.IsNullOrEmpty(Picklist)) {
				throw new Exception("Picklist number must be set in app.config");
			}
		}

		private static SAPbobsCOM.Company ConnectToCompany() {
			SAPbobsCOM.Company company = new SAPbobsCOM.Company();
			company.Server = ServerName;
			company.CompanyDB = DatabaseName;
			company.UserName = SapUserName;
			company.Password = SapPassword;
			company.UseTrusted = false;
			switch (ServerType) {
				case "HANA": company.DbServerType = SAPbobsCOM.BoDataServerTypes.dst_HANADB; break;
				case "SQL2008": company.DbServerType = SAPbobsCOM.BoDataServerTypes.dst_MSSQL2008; break;
				case "SQL2012": company.DbServerType = SAPbobsCOM.BoDataServerTypes.dst_MSSQL2012; break;
				case "SQL2014": company.DbServerType = SAPbobsCOM.BoDataServerTypes.dst_MSSQL2014; break;
				case "SQL2016": company.DbServerType = SAPbobsCOM.BoDataServerTypes.dst_MSSQL2016; break;
				default: throw new Exception("Invalid ServerType '" + ServerType + "' in app.config");
			}
			company.DbUserName = DatabaseUserName;
			company.DbPassword = DatabasePassword;
			int res = company.Connect();
			if (res != 0) {
				int err;
				string error;
				company.GetLastError(out err, out error);
				throw new Exception(" Connection to company failed: " + err.ToString() + " " + error);
			} else {
				Console.WriteLine(" Connection to company successful.");
			}
			return company;
		}
	}


}
